import React, {Component, PropTypes} from 'react';
import SyntaxHighlighter, {registerLanguage} from 'react-syntax-highlighter/dist/light';
import {createElement} from 'react-syntax-highlighter';

import python from 'react-syntax-highlighter/dist/languages/python';

import * as styles from 'react-syntax-highlighter/dist/styles';

import * as R from 'ramda';

const AVAILABLE_STYLES = [
    'androidstudio', 'arduinoLight', 'arta', 'ascetic', 'atelierCaveDark',
    'atelierCaveLight', 'atelierDuneDark', 'atelierDuneLight',
    'atelierEstuaryDark', 'atelierEstuaryLight',
    'atelierForestDark', 'atelierForestLight', 'atelierHeathDark',
    'atelierHeathLight', 'atelierLakesideDark', 'atelierLakesideLight',
    'atelierPlateauDark', 'atelierPlateauLight', 'atelierSavannaDark',
    'atelierSavannaLight', 'atelierSeasideDark', 'atelierSeasideLight',
    'atelierSulphurpoolDark', 'atelierSulphurpoolLight', 'atomOneDark',
    'atomOneLight', 'brownPaper', 'codepenEmbed', 'colorBrewer',
    'darcula', 'dark', 'darkula', 'defaultStyle', 'docco', 'dracula',
    'far', 'foundation', 'githubGist', 'github', 'googlecode', 'grayscale',
    'gruvboxDark', 'gruvboxLight', 'hopscotch', 'hybrid', 'idea', 'irBlack',
    'kimbieDark', 'kimbieLight', 'magula', 'monoBlue', 'monokaiSublime',
    'monokai', 'obsidian', 'ocean', 'paraisoDark', 'paraisoLight',
    'pojoaque', 'purebasic', 'qtcreatorDark', 'qtcreatorLight',
    'railscasts', 'rainbow', 'routeros', 'schoolBook', 'solarizedDark',
    'solarizedLight', 'sunburst', 'tomorrowNightBlue', 'tomorrowNightBright',
    'tomorrowNightEighties', 'tomorrowNight', 'tomorrow', 'vs',
    'vs2015', 'xcode', 'xt256', 'zenburn'
];

registerLanguage('python', python);

class AnnotatedCode extends Component {
    constructor() {
        super();
        this.state = {
            comments: []
        };
    }

    componentDidMount() {
        const {code, theme, styles_syntax, styles_syntax_span} = this.props;
        const syntax = (
            <SyntaxHighlighter
                language={'python'}
                style={styles[theme]}
                wrapLines={true}
                customStyle={R.merge({
                    'paddingLeft': '10px',
                    'paddingTop': 0,
                    'margin': 0
                }, styles_syntax)}
                renderer={({rows, stylesheet, useInlineStyles}) => {
                    const comments = Array.from(
                        Array(rows.length),
                        () => '\n'
                    );
                    const renderedRows = rows.map((row, i) => {
                        row.children.forEach((snippet, j) => {
                            if (R.has('properties', snippet) &&
                                R.contains('hljs-comment',
                                           snippet.properties.className)
                            ) {
                                comments[i] = (
                                    snippet.children[0].value.replace('# ', '')
                                );
                                delete row.children[j];
                            }
                        });
                        const style=R.merge(
                            {'display': 'block'},
                            styles_syntax_span
                        );
                        return createElement({
                            node: row,
                            stylesheet,
                            useInlineStyles,
                            style
                        })
                    });
                    this.setState({comments});
                    return renderedRows;
                }}
            >
                {code}
            </SyntaxHighlighter>
        );
        this.setState({syntax});
    }

    render() {
        const {
            styles_container,
            styles_syntax_container,
            styles_comment_container,
            styles_comment_pre,
            styles_comment_span
        } = this.props;
        return (<div style={R.merge({
                'width': '1000px',
                'marginLeft': 'auto',
                'marginRight': 'auto',
                'marginTop': 20,
                'border': 'thin rgb(233, 233, 255) solid',
                'display': 'flex'
            }, styles_container)}>
                <div style={R.merge({
                    'width': '500px',
                    'order': 1,
                    'flex': 1,
                    'flexBasis': 'auto'
                }, styles_syntax_container)}>
                    {this.state.syntax}
                </div>

                <div style={R.merge({
                    'order': 1,
                    'flex': 5,
                    'marginRight': 10,
                    'paddingLeft': 10
                }, styles_comment_container)}>
                    <pre style={R.merge({
                        backgroundColor: 'rgb(255, 255, 255)',
                        color: '#506784',
                        margin: 0,
                        padding: 0
                    }, styles_comment_pre)}>
                        <code>
                            {this.state.comments.map(comment => {
                                const style = R.merge(
                                    {'display': 'block'},
                                    styles_comment_span
                                );
                                return (
                                    <span style={style}>
                                        {comment}
                                    </span>
                                )
                            })}
                        </code>
                    </pre>
                </div>
        </div>);
    }
}

AnnotatedCode.propTypes = {
    code: PropTypes.string,
    theme: PropTypes.oneOf(AVAILABLE_STYLES),

    styles_container: PropTypes.object,

    styles_syntax_container: PropTypes.object,
    styles_syntax: PropTypes.object,
    styles_syntax_span: PropTypes.object,

    styles_comment_container: PropTypes.object,
    styles_comment_pre: PropTypes.object,
    styles_comment_span: PropTypes.object
}

export default AnnotatedCode;
