import dash_annotated_syntax_highlighter
import dash
import dash_html_components as html

app = dash.Dash('')

app.scripts.config.serve_locally = True

code = '''
import dash
import dash_html_components as html # includes all (510!) html tags and their properties
import dash_core_components as dcc # widget library: dropdowns, graphs, sliders, tabs, etc


app = dash.Dash()

app.layout = html.Div(children=[ # \`app.layout\` describes
                                 # what the app looks like


    html.H1(children='Hello Dash'), # H1 a header.
    # See also H2, H3, ..., H6

    html.Div(children="Dash: A web application framework for Python."),

    dcc.Graph( # dcc.Graph uses the plotly.js library
        id='example-graph',
        figure={
            'data': [
                {
                    'x': [1, 2, 3],
                    'y': [4, 1, 2],
                    'type': 'bar',
                    'name': 'SF'
                },
                {
                    'x': [1, 2, 3],
                    'y': [2, 4, 5],
                    'type': 'bar',
                    'name': 'Montreal'
                },
            ],
            'layout': { # describes titles, axes, annotations, legends, and more
                'title': 'Dash Data Visualization'
            }
        }
    )
])

if __name__ == '__main__':
    app.run_server(debug=True) # debug=True will reload the app when you make changes to the code
'''

app.layout = html.Div([
    dash_annotated_syntax_highlighter.AnnotatedSyntaxHighlighter(
        code=code,
        theme='hopscotch'
    )
])


if __name__ == '__main__':
    app.run_server(debug=True)
